/* Design for the fashion system 2020/21 
Fashion 4.0
Group 2
HYPE Project - Hear Your Personal Emotions */


// We include the required libraries
#include "Arduino.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"

//--------------------GLOBAL VARIABLES----------------------------//

  // Definition of PINs
  int PulseSensorPin = A3;  
  int Button = 13; 
  int Green_light = 3;
  
  // Variables
  int counter = 0;
  int button_value = 0;
  int threshold = 525;
  int beat = 0;
  int beat_counter = 0;
  int sum = 0;

  double aux = 0;
  double result = 0;
  double bpm[] = {0,0,0,0,0,0,0,0,0,0};
  
  unsigned long previousMillis = 0;

  // Define Serial Connection and MP3 
  SoftwareSerial mySoftwareSerial(10, 11); // RX, TX
  DFRobotDFPlayerMini myDFPlayer; // Create an instance of DFRobotDFPlayerMini class

//--------------------------SETUP--------------------------------//

void setup() {
  
  pinMode(PulseSensorPin, INPUT); // Signal received of the pulse
  pinMode(Button, INPUT_PULLUP);  // Signal received from button 
                                  // (INPUT_PULLUP fixes Button instability)
  pinMode(Green_light, OUTPUT);   // We have to turn on the LED

  // We define different Serial ports to keep track of the state of the variables
  mySoftwareSerial.begin(9600);
  Serial.begin(115200);
  
  Serial.println();
  Serial.println(F("Welcome to HYPE!"));
  Serial.println(F("Hear Your Personal Emotions"));
  
  // Use softwareSerial to communicate with mp3
  if (!myDFPlayer.begin(mySoftwareSerial)) {  
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
    while(true);
  }
  Serial.println(F("Initializing..."));
  
  // Set volume value. From 0 to 30 adjust it to your situation
  myDFPlayer.volume(30);  
  
}

//--------------------------LOOP---------------------------------//

void loop() {

  // Measure device time in milliseconds
  unsigned long currentMillis = millis();
  // Records the signal emited from the pulse sensor
  beat = analogRead(PulseSensorPin);
  
  // If the beat exceeds a certain threshold record a beat
  if(beat > threshold){
    beat_counter++;
  }
  
  // Each 6 seconds records the beats and makes a 60 seconds estimation (BPM)
  if((unsigned long)(currentMillis-previousMillis) >= 6000){
    aux = (60/((unsigned long)(((currentMillis-previousMillis)/1000))))*beat_counter;
    bpm[counter] = aux;

    // Keep track of the values in the Serial Monitor
    Serial.print("Time stamp: ");
    Serial.print(currentMillis/1000);
    Serial.println(" seconds");
    Serial.print("Beat counter: ");
    Serial.println(beat_counter);
    Serial.print("Expected BPM: ");
    Serial.println(aux);

    beat_counter = 0;
    counter++;
    previousMillis = currentMillis;
  }
  /* Once a minute has passed we average the values recorded, 
   *  this way we are able to deal with some inconsistencies
   *  of the sensor.
   */
  if(counter == 10){
    for(int i=0; i<10; i++){
      sum += bpm[i];
      Serial.print("Beat ");
      Serial.print(i);
      Serial.print(" ");
      Serial.println(bpm[i]);       
    }
    result = sum/10;
    Serial.print("Average: ");
    Serial.println(result);
    counter=0;
  }
  button_value = digitalRead(Button);
  // Auxiliar method
  play_music();

  // Turns ON the green led once a minute has passed
  if(currentMillis>=60000){
    digitalWrite(Green_light, HIGH);
  }
}

//--------------------------AUXILIAR METHOD---------------------//

/* Plays the desired music depending on the averaged beat.
   This method can be enhanced by adding more if else statements.
   However this sums up the desired output that we wanted in a 
   simple way. */
   
void play_music(){
  if(!button_value == 1 && digitalRead(Green_light)){
    if(result < 70){
      myDFPlayer.play(1);
    }else{
      myDFPlayer.play(2);
    }
  }
}
